//// DropView.cpp
//

#define DEBUG 0
#include <Debug.h>

#include <Application.h>
#include <Roster.h>
#include <Window.h>
#include <Box.h>
#include <TextControl.h>
#include <Button.h>
#include <CheckBox.h>
#include <Node.h>
#include <NodeInfo.h>
#include <Bitmap.h>
#include <String.h>
#include <Beep.h>
#include <fs_attr.h>


#include "Defs.h"
#include "EditView.h"

EditView::EditView(entry_ref a_ref)
: BBox(BRect(5,5,330,135), "EditView", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS)
{
	m_ref	=	a_ref;

	float str_width	=	StringWidth("Working dir");
	
	current_dir	=	new BTextControl(BRect(5,60,318,80), "Working dir", "Working dir", "", NULL, B_FOLLOW_LEFT_RIGHT);
	arguments	=	new BTextControl(BRect(5,85,318,105), "Arguments", "Arguments", "", NULL, B_FOLLOW_LEFT_RIGHT);

	current_dir->	SetDivider(str_width-5);
	arguments->		SetDivider(str_width-5);
	
	m_cancel	=	new BButton(BRect(93,115,205,140), "cancelbutton", "Cancel", new BMessage(RIT_CANCEL), B_FOLLOW_RIGHT);
	m_save	=	new BButton(BRect(210,115,318,140), "savebutton", "Save", new BMessage(RIT_SAVE_REQUESTED), B_FOLLOW_RIGHT);
	
	m_save->MakeDefault(true);
	
	keep_open	=	new BCheckBox (BRect(5,115,100,140), "keep_open", "Keep open", NULL);
	keep_open->SetValue(true);
	
	AddChild(current_dir);
	AddChild(arguments);
	AddChild(m_cancel);
	AddChild(m_save);
	AddChild(keep_open);
	
	str_width			=	StringWidth("Open in Terminal");
	float str_height	=	StringWidth("M");
	m_open_in_terminal	=	new BCheckBox (BRect(0,0,str_width+10,str_height+10), 
							"Open in Terminal", "Open in Terminal", new BMessage(RIT_OPEN_IN_TERMINAL));
	
}

void 
EditView::AllAttached	(void)
{
	m_cancel->				SetTarget(this);
	m_save->				SetTarget(this);
	m_open_in_terminal->	SetTarget(this);

	SetLabel(m_open_in_terminal);
	
	Init();
}

void 
EditView::Draw	(BRect updateRect)	
{
	SetHighColor(Parent()->ViewColor());
	FillRect(updateRect);

	SetHighColor(0,0,0);

	float str_width		=	StringWidth("Working dir");
	float str_height	=	StringWidth("W");
	float name_width	=	StringWidth(m_ref.name);

	
	MovePenTo((str_width + (Bounds().Width()-(str_width))/2 - (name_width/2) - 7),
				(56 - 15 + (str_height/2) - 1));

	DrawString(m_ref.name);

	SetDrawingMode(B_OP_OVER);
	
	if (m_bitmap != NULL)
		DrawBitmap(m_bitmap, BRect((str_width/2-15), 25, (str_width/2+16), 56));
	else
	{
		// Grey fill
		SetHighColor(Parent()->ViewColor());
		FillRect(BRect((str_width/2-15), 5, (str_width/2+16), 36));
	}

	BBox::Draw(updateRect);
}

void 
EditView::FrameResized	(float width, float height)	
{
	BBox::FrameResized(width, height);

	Draw(Bounds());
}

EditView::~EditView()	
{
	delete m_bitmap;
}

void 
EditView::MessageReceived(BMessage * a_message)
{
	switch(a_message->what)
	{
		case RIT_SAVE_REQUESTED:
				Save();
				a_message->what = RIT_CANCEL;
				BBox::MessageReceived(a_message);
				break;
				
		case RIT_OPEN_IN_TERMINAL:
				OpenInTerminalOnOff();
				break;
				
		default:
				BBox::MessageReceived(a_message);
			break;			
	}
}

void 
EditView::Save	(void)
{
	BNode		node		(& m_ref);
	off_t		bytes	=	0;	
	
	BString	args	=	arguments->Text();
	BString	dir		=	current_dir->Text();
	
	if (args.Length() != 0)
	{
		bytes = node.WriteAttr("RunInTerminal:Args", B_STRING_TYPE, 0, args.String(), args.Length()+1);
		if (! bytes == args.Length()+1)
			beep();
	}
	else
		node.RemoveAttr("RunInTerminal:Args");
	
	if (dir.Length() != 0)
	{
		bytes = node.WriteAttr("RunInTerminal:Dir", B_STRING_TYPE, 0, dir.String(), dir.Length()+1);
		if (! bytes == dir.Length()+1)
			beep();
	}
	else
		node.RemoveAttr("RunInTerminal:Dir");
	
	if (! keep_open->Value())
		bytes = node.WriteAttr("RunInTerminal:Close", B_STRING_TYPE, 0, "Y", 2);
	else
		node.RemoveAttr("RunInTerminal:Close");
}

void 
EditView::Init	(void)
{
	BNode		node		(& m_ref);
	BNodeInfo	nodeinfo	(& node);
	
	m_bitmap	=	new BBitmap	(BRect(0,0,31,31), B_COLOR_8_BIT, true);
		
	if (BNodeInfo::GetTrackerIcon(& m_ref, m_bitmap, B_LARGE_ICON) != B_OK)
		m_bitmap = NULL;

	char * buffer	=	new char [4096];
	off_t	bytes	=	0;
	
	bytes = node.ReadAttr("RunInTerminal:Args", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		arguments->SetText(buffer);
	}
	
	bytes = node.ReadAttr("RunInTerminal:Dir", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		current_dir->SetText(buffer);
	}
	
	attr_info	info;
	if (node.GetAttrInfo("RunInTerminal:Close", & info) == B_OK)
		keep_open->SetValue(false);
	
	buffer[0] = '\0';
	bytes = node.ReadAttr("BEOS:PREF_APP", B_MIME_TYPE, 0, buffer, 4095);

	if (bytes >= 0)
		buffer[bytes] = '\0';
	//else
	//	PRINT(("APP_SIG: %s\n", strerror(bytes)));
		
	BString app_sig (buffer);
	
	if (app_sig == RIT_APP_SIG)
	{
		m_open_in_terminal->	SetValue(true);
		current_dir->			SetEnabled(true);
		arguments->				SetEnabled(true);
		m_cancel->				SetEnabled(true);
		m_save->				SetEnabled(true);
		keep_open->				SetEnabled(true);
	}
	else
	{
		m_open_in_terminal->	SetValue(false);
		current_dir->			SetEnabled(false);
		arguments->				SetEnabled(false);
		m_cancel->				SetEnabled(false);
		m_save->				SetEnabled(false);
		keep_open->				SetEnabled(false);
	}
	
	delete [] buffer;
}

void 
EditView::OpenInTerminalOnOff	(void)
{
	BNode	node (& m_ref);

	if (current_dir->IsEnabled())
	{
		current_dir->			SetEnabled(false);
		arguments->				SetEnabled(false);
		m_cancel->				SetEnabled(false);
		m_save->				SetEnabled(false);
		keep_open->				SetEnabled(false);
		
		node.RemoveAttr("BEOS:PREF_APP");
	}
	else
	{
		current_dir->			SetEnabled(true);
		arguments->				SetEnabled(true);
		m_cancel->				SetEnabled(true);
		m_save->				SetEnabled(true);
		keep_open->				SetEnabled(true);
		
		node.WriteAttr("BEOS:PREF_APP", B_MIME_TYPE, 0, RIT_APP_SIG, 40);
	}
}
