#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <String.h>
#include <Path.h>
#include <Entry.h>
#include <FindDirectory.h>
#include <Directory.h>
#include <Node.h>
#include <fs_attr.h>

#include "RunInTerminalApp.h"

RunInTerminalApp::RunInTerminalApp	(void)
 : BApplication	("application/x-vnd.Kirilla-RunInTerminal")	
{
	argv_received	=	false;
	refs_received	=	false;
	
	if (CreateTempFolder() != B_OK)
		exit(-1);	

	// read settings
	// ... 

	m_window = 	new FavoriteWindow (BRect(300,200,495,320));
}

RunInTerminalApp::~RunInTerminalApp	(void)	
{
	// void
}

void 
RunInTerminalApp::ArgvReceived	(int32 argc, char **argv)
{
	argv_received	=	true;

	BMessage *	msg = new BMessage(B_REFS_RECEIVED);
	int32	result;

	if(argc <= 1)	// no arguments
		return;

//	get current path

	char *	buffer = new char[4096];
	getcwd(buffer, 4096);
	BString	currentdir = buffer;
	if (currentdir.FindLast('/') != currentdir.Length()-1)
		currentdir.Append("/");
	delete buffer;

//	convert arguments to entry_refs

	for(int32 i = 1; i < argc; i++)
	{
		BString path_str = argv[i];
		result =  path_str.FindFirst("/");
		
		if (result == B_ERROR || result != 0)
		{
			path_str.Prepend(currentdir);
		}
			
		BEntry		entry(path_str.String());
		entry_ref	ref;
		entry.GetRef(& ref);
		
		msg->AddRef("refs", & ref);

	}
	
//	Send to RefsReceived
	RefsReceived(msg);
	delete msg;
}

void 
RunInTerminalApp::RefsReceived	(BMessage * a_message)
{
	refs_received	=	true;
	
	ProcessRefs(a_message);
}

void
RunInTerminalApp::ReadyToRun	(void)
{
	if (!argv_received && !refs_received)
	{
		m_window->Show();
		return;
		

	}

	PostMessage(B_QUIT_REQUESTED);
}

void 
RunInTerminalApp::MessageReceived	(BMessage * a_message)
{
	switch(a_message->what)
	{
		case RIT_HELP_REQUESTED:
				Help();
				break;
				
		default:
				BApplication::MessageReceived(a_message);
				break;
	}
}

void
RunInTerminalApp::AboutRequested	(void)
{
	BAlert * alert = new BAlert("RunInTerminal", "RunInTerminal 1.2\n\n"
	"Freeware   " B_UTF8_COPYRIGHT " Jonas Sundström, Kirilla.com\n", "Kirilla", "Ok",
	NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT);

	int32 reply = alert->Go();
	
	if (reply == 0)
	{
		BMessage msg(B_ARGV_RECEIVED); 
		msg.AddString("argv", "app"); 
		msg.AddString("argv", "http://www.kirilla.com/");  
		msg.AddInt32("argc", 2); 
		be_roster->Launch ("application/x-vnd.Be.URL.http", &msg ); 			
	}
}

void 
RunInTerminalApp::Help	(void)
{
	(new BAlert("RunInTerminal", "You can use RunInTerminal in 3 ways:\n"
	"1. Drop scripts/apps on the icon of RunInTerminal\n"
	"2. Select RunInTerminal in Tracker's Open With...\n"
	"3. Set your individual script/app files to open with RunInTerminal "
	"(using the FileType addon)\n\n"
	
	"You can specify preferences "
	"(working directory, arguments and keep-open) "
	"on a per-file basis, by dropping your script/app "
	"in RunInTerminal's window. "
	"It has an editor built into it.", "I see", 
	NULL, NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT))->Go();
}

void 
RunInTerminalApp::ProcessRefs	(BMessage * a_message)
{
	entry_ref	ref;
	int32		ref_count	=	0;

	while(a_message->FindRef("refs", ref_count, &ref) == B_OK)
	{
		RunScript(& ref);
		ref_count++;	
	}
}

void 
RunInTerminalApp::RunScript(const entry_ref * a_ref)	
{
	// read setting attributes from app/script
	BString	args;
	BString	dir;
	
	BNode	node	(a_ref);
	char *	buffer	=	new char [4096];
	off_t	bytes	=	0;
	
	bytes = node.ReadAttr("RunInTerminal:Args", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		args =	buffer;
	}
	
	bytes = node.ReadAttr("RunInTerminal:Dir", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		dir =	buffer;
	}
	
	delete [] buffer;

	// create kickstart_script
	BPath	path	(a_ref);
	BString	path_string		(path.Path());
	BString	name_string		(a_ref->name);
	FullQuote(& name_string);
	FullQuote(& path_string);

	BString kickstart_script	("#!/bin/sh\n");

	// working dir
	kickstart_script.Append("cd ");	
	kickstart_script.Append(dir);	
	kickstart_script.Append("\n");	

	// app/script
	kickstart_script.Append(path_string);
	kickstart_script.Append(" ");
	kickstart_script.Append(args);	// arguments
	kickstart_script.Append("\n");
	
	// keep open
	attr_info	info;
	if (node.GetAttrInfo("RunInTerminal:Close", & info) != B_OK)
		kickstart_script.Append("read -p \"\n(Press Enter to quit)\"\n");
	
	//	create tempfile name
	int32	counter	=	1;
	BString tempfile_name;
	tempfile_name << counter;
	while(m_tmp_dir->Contains(tempfile_name.String()))
	{
		counter	++;
		tempfile_name.Truncate(0);
		tempfile_name << counter;
	}
	
	// write kickstart script to file
	BFile	tmp_file;
	if (m_tmp_dir->CreateFile(tempfile_name.String(), & tmp_file) != B_OK)
		exit(-1);
		
	tmp_file.Write(kickstart_script.String(), kickstart_script.Length());
	
	// open Terminal with kickstart script
	
	BPath	tempfile_path	(m_tmp_dir, tempfile_name.String());
	BString command_string	(tempfile_path.Path());
	NameSafe(& command_string);

	command_string.Prepend(" ");
	command_string.Prepend(name_string);
	command_string.Prepend("/boot/beos/apps/Terminal -t ");
	command_string.Append(" &");
		
	system(command_string.String());	
}

void
RunInTerminalApp::NameSafe	(BString * a_string)
{
	int32	position = 0;
	while(position < a_string->	Length())
	{
		if      (a_string-> ByteAt (position) == '\"') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '`') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '$') { a_string-> Insert ("\\",position); position++; }
		position++;
	}
	
	a_string->Prepend("\"");
	a_string->Append("\"");
	
}

void
RunInTerminalApp::FullQuote	(BString * a_string)
{
	int32	position = 0;
	while(position < a_string->	Length())
	{
		if      (a_string-> ByteAt (position) == ';') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '&') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '(') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == ')') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '|') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '<') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '>') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '*') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '?') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '[') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == ']') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '~') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '@') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '!') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '"') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '\'') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '\\') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '`') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '$') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == ' ') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '{') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '}') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '#') { a_string-> Insert ("\\",position); position++; }
		position++;
	}
}

status_t
RunInTerminalApp::CreateTempFolder	(void)
{
	status_t status;
	BPath p;
	if (find_directory(B_COMMON_TEMP_DIRECTORY, &p) != B_OK)	return B_ERROR;
	
	m_tmp_dir	=	new BDirectory();
	BDirectory temp_dir;
	
	if ((status = m_tmp_dir->	SetTo(p.Path())) != B_OK) 	return status;
	status	=	m_tmp_dir->	CreateDirectory("Kirilla RunInTerminal", &temp_dir);
	if (status != B_OK && status != B_FILE_EXISTS)	return status;

	p.SetTo(p.Path(),"Kirilla RunInTerminal");
	if ((status = m_tmp_dir->	SetTo(p.Path())) != B_OK)	return status;
		
	BEntry entry(p.Path());
	if (entry.Exists()	&&	entry.IsDirectory())	return B_OK;
	else											return B_ERROR;
}
